package de.jpct.proto;

import java.util.ArrayList;
import java.util.List;

import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.RGBColor;
import com.threed.jpct.World;

public class FlagPool {

	private final static Object3D BLUEPRINT;
	private World world = null;
	private int cnt = 0;

	static {
		BLUEPRINT = Primitives.getPlane(1, 3.5f);
		BLUEPRINT.setTexture("flag");
		BLUEPRINT.getRotationMatrix().set(1, 1, 2);
		BLUEPRINT.rotateMesh();
		BLUEPRINT.clearRotation();
		BLUEPRINT.build();
		BLUEPRINT.setBillboarding(true);
		BLUEPRINT.setTransparency(40);
		BLUEPRINT.translate(0, -3, 0);
		BLUEPRINT.setAdditionalColor(RGBColor.WHITE);
		
	}

	private List<Object3D> flags = new ArrayList<Object3D>();

	public FlagPool(World world) {
		this.world = world;
		createNew(50);
	}

	public void clear() {
		int end = flags.size();
		for (int i = 0; i < end; i++) {
			Object3D flag = flags.get(i);
			flag.setVisibility(false);
			flag.setUserObject(null);
		}
		cnt = 0;
	}

	public void clear(GridPosition gp) {
		for (int i = 0; i < cnt; i++) {
			Object3D flag = flags.get(i);
			if (flag.getUserObject().equals(gp)) {
				flag.setVisibility(false);
			}
		}
	}

	public void add(GridPosition gp) {
		if (cnt >= flags.size()) {
			createNew(20);
		}
		Object3D flag = flags.get(cnt);
		flag.setVisibility(true);
		flag.clearTranslation();
		flag.translate(gp.convertTo3D());
		flag.translate(0, -3, 0);
		flag.setUserObject(gp);
		cnt++;
	}

	private void createNew(int number) {
		for (int i = 0; i < number; i++) {
			Object3D flag = new Object3D(BLUEPRINT, true);
			flag.shareCompiledData(BLUEPRINT);
			world.addObject(flag);
			flags.add(flag);
			flag.setVisibility(false);
			flag.setLighting(Object3D.LIGHTING_NO_LIGHTS);
		}
	}
}
