package de.jpct.proto;

import java.util.List;

import android.content.res.Resources;

import com.threed.jpct.*;

/**
 * An Object3D that represents the actual level. This example "game" only has
 * one level. If you want more, you'll have to implement a kind of level
 * management yourself.
 */
public class Level {

	static final private long serialVersionUID = 1L;
	/*
	private final static int WALL_OCTREE_SIZE = 100;
	private final static int FLOOR_OCTREE_SIZE = 100;
	private final static int TOP_OCTREE_SIZE = 100;
	*/

	private LevelPart floor = null;
	private LevelPart top = null;
	private LevelPart walls = null;
	private MapMask mask = null;

	public Level(Resources res) throws Exception {

		List<?> parts = LevelLoader.loadMap(res);
		walls = (LevelPart) parts.get(0);
		top = (LevelPart) parts.get(1);
		floor = (LevelPart) parts.get(2);
		mask = (MapMask) parts.get(3);

		walls.enableLazyTransformations();
		walls.getMesh().compress();
		walls.build();
		/*
		OcTree ocw = new OcTree(walls.getMesh(), WALL_OCTREE_SIZE, OcTree.MODE_OPTIMIZED);
		ocw.setCollisionUse(OcTree.COLLISION_USE);
		walls.setOcTree(ocw);
		*/
		walls.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);
		walls.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);

		top.enableLazyTransformations();
		floor.enableLazyTransformations();

		top.getMesh().compress();
		floor.getMesh().compress();

		top.build();
		floor.build();
		top.setLighting(Object3D.LIGHTING_NO_LIGHTS);
		top.setAdditionalColor(new RGBColor(50, 50, 50));
		/*
		OcTree ocf = new OcTree(floor.getMesh(), FLOOR_OCTREE_SIZE, OcTree.MODE_OPTIMIZED);
		OcTree oct = new OcTree(top.getMesh(), TOP_OCTREE_SIZE, OcTree.MODE_OPTIMIZED);

		ocf.setCollisionUse(OcTree.COLLISION_USE);
		oct.setCollisionUse(OcTree.COLLISION_USE);

		floor.setOcTree(ocf);
		top.setOcTree(oct);
		*/
	}

	public void addToWorld(World world) {
		world.addObject(walls);
		if (floor != null) {
			world.addObject(floor);
		}
		if (top != null) {
			world.addObject(top);
		}
	}

	public void addCollisionListener(CollisionListener colLis) {
		walls.addCollisionListener(colLis);
	}

	public int getWidth() {
		return mask.getWidth();
	}

	public int getHeight() {
		return mask.getHeight();
	}

	public SimpleVector getCenter() {
		return floor.getTransformedCenter();
	}

	public Camera createAllViewingCamera(SimpleVector position, FrameBuffer buffer) {
		Camera cam = new Camera();
		try {
			cam.setFOVLimits(0, 100);
			cam.setFOV(1);
			cam.setYFOV(1);

			SimpleVector midPoint = floor.getTransformedCenter();

			// midPoint.x=(position.x+midPoint.x*(getWidth()-6))/(getWidth()-5);
			// midPoint.z=(position.z+midPoint.z*(getHeight()-6))/(getHeight()-5);

			midPoint.x = (position.x + midPoint.x * 10f) / 11f;
			midPoint.z = (position.z + midPoint.z * 10f) / 11f;

			cam.setPosition(position);
			cam.lookAt(midPoint);

			float[] bb = walls.getMesh().getBoundingBox();

			// minX, maxX, minY, maxY, minZ, maxZ
			SimpleVector c1 = new SimpleVector(bb[1], bb[3], bb[4]);
			SimpleVector c2 = new SimpleVector(bb[1], bb[3], bb[5]);
			SimpleVector c3 = new SimpleVector(bb[0], bb[3], bb[4]);
			SimpleVector c4 = new SimpleVector(bb[0], bb[3], bb[5]);

			SimpleVector c1s = Interact2D.project3D2D(cam, buffer, c1);
			SimpleVector c2s = Interact2D.project3D2D(cam, buffer, c2);
			SimpleVector c3s = Interact2D.project3D2D(cam, buffer, c3);
			SimpleVector c4s = Interact2D.project3D2D(cam, buffer, c4);

			// System.out.println(c1s.x+"/"+c1s.y+ " - "+ c2s.x+"/"+c2s.y+
			// " - "+c3s.x+"/"+c3s.y+ " - "+c4s.x+"/"+c4s.y);

			float ymin = 999999;
			float ymax = -999999;
			float xmin = 999999;
			float xmax = -999999;

			if (c1s != null) {
				if (c1s.x < xmin)
					xmin = c1s.x;
				if (c1s.x > xmax)
					xmax = c1s.x;
				if (c1s.y < ymin)
					ymin = c1s.y;
				if (c1s.y > ymax)
					ymax = c1s.y;
			}

			if (c2s != null) {
				if (c2s.x < xmin)
					xmin = c2s.x;
				if (c2s.x > xmax)
					xmax = c2s.x;
				if (c2s.y < ymin)
					ymin = c2s.y;
				if (c2s.y > ymax)
					ymax = c2s.y;
			}

			if (c3s != null) {
				if (c3s.x < xmin)
					xmin = c3s.x;
				if (c3s.x > xmax)
					xmax = c3s.x;
				if (c3s.y < ymin)
					ymin = c3s.y;
				if (c3s.y > ymax)
					ymax = c3s.y;
			}

			if (c4s != null) {
				if (c4s.x < xmin)
					xmin = c4s.x;
				if (c4s.x > xmax)
					xmax = c4s.x;
				if (c4s.y < ymin)
					ymin = c4s.y;
				if (c4s.y > ymax)
					ymax = c4s.y;
			}

			xmin = (buffer.getWidth() / 2) - xmin;
			xmax = xmax - (buffer.getWidth() / 2);

			ymin = (buffer.getHeight() / 2) - ymin;
			ymax = ymax - (buffer.getHeight() / 2);

			float xd = xmin;
			if (xmax > xd) {
				xd = xmax;
			}

			float yd = ymin;
			if (ymax > yd) {
				yd = ymax;
			}

			float xfov = xd / (buffer.getWidth() / 2);
			float yfov = yd / (buffer.getHeight() / 2);

			cam.setFOV(xfov);
			cam.setYFOV(yfov);

		} catch (Exception e) {
			e.printStackTrace();
		}

		return cam;
	}

	/**
	 * Returns a simple checksum to ensure (well, not really...) that all
	 * clients are using the same level. This has to be improved for "real"
	 * games to prevent cheating. For this example, it's fine the way it is.
	 * 
	 * @return int the checksum
	 */
	public int getChecksum() {
		return walls.getMesh().getTriangleCount() * walls.getMesh().getUniqueVertexCount() + walls.getMesh().getVertexCount();
	}

	public MapMask getMask() {
		return mask;
	}

	public boolean wasCollisionParticipant() {
		return walls.wasTargetOfLastCollision();
	}
}