package de.jpct.proto;

import android.content.res.Resources;

import com.threed.jpct.Camera;
import com.threed.jpct.Loader;
import com.threed.jpct.Object3D;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;

public class Player extends Object3D {

	private static final long serialVersionUID = 1L;
	private float zoomFactor = 1f;

	private SimpleVector oldestCamPos = new SimpleVector();
	private SimpleVector camPos = new SimpleVector();
	private SimpleVector yOffset = new SimpleVector();
	private SimpleVector zAxis = new SimpleVector();
	private SimpleVector direction = new SimpleVector(0, 0, 1);

	public Player(Resources res) {
		super(Object3D.mergeAll(Loader.load3DS(res.openRawResource(R.raw.playerobj), 0.075f)));
		setTexture("player");
		setAdditionalColor(new RGBColor(50, 50, 50));
		build();
	}

	public void zoom(float add) {
		zoomFactor += add;
	}

	public void rotateDirection(float val) {
		direction.rotateY(val);
	}
	
	public void alignCamera(Camera camera, int ticks) {
		SimpleVector trns = getTranslation();

		rotateY(0.02f * ticks);

		for (int i = 0; i < ticks; i++) {

			// Making this interpolation based on ticks fixes the jitter on low
			// fps
			SimpleVector center = trns;

			SimpleVector oldCamPos = camera.getPosition();
			oldestCamPos.set(oldCamPos);

			camPos.set(center);
			zAxis.set(direction);

			yOffset.set(0, -70 * zoomFactor * zoomFactor, 0);
			zAxis.scalarMul(-70f * zoomFactor);

			camPos.add(zAxis);
			camPos.add(yOffset);

			oldestCamPos.scalarMul(-1f);

			camPos.add(oldestCamPos);
			float len = camPos.length();

			camPos.x /= len;
			camPos.y /= len;
			camPos.z /= len;

			if (len != 0) {
				camera.moveCamera(camPos, len);
			}
			camera.lookAt(center);
		}
	}
}
