package de.jpct.proto;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.Window;
import android.view.WindowManager.LayoutParams;

import com.threed.jpct.Camera;
import com.threed.jpct.Config;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Interact2D;
import com.threed.jpct.Light;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;

public class Prototype extends Activity {

	private GLSurfaceView mGLView;
	private MyRenderer renderer = null;
	private World world = null;
	private float zoom = 0;
	private float turn = 0;

	private float touchTurn = 0;
	private float touchTurnUp = 0;

	private float xpos = -1;
	private float ypos = -1;

	protected void onCreate(Bundle savedInstanceState) {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(LayoutParams.FLAG_FULLSCREEN, LayoutParams.FLAG_FULLSCREEN);

		super.onCreate(savedInstanceState);
		mGLView = new GLSurfaceView(getApplication());

		mGLView.setEGLConfigChooser(new GLSurfaceView.EGLConfigChooser() {
			public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display) {
				int[] attributes = new int[] { EGL10.EGL_DEPTH_SIZE, 16, EGL10.EGL_NONE };
				EGLConfig[] configs = new EGLConfig[1];
				int[] result = new int[1];
				egl.eglChooseConfig(display, attributes, configs, 1, result);
				return configs[0];
			}
		});
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		renderer = new MyRenderer();
		mGLView.setRenderer(renderer);
		setContentView(mGLView);
	}

	@Override
	protected void onPause() {
		super.onPause();
		mGLView.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();
		mGLView.onResume();
	}

	protected void onStop() {
		super.onStop();
		renderer.stop();
	}

	public boolean onTouchEvent(MotionEvent me) {

		if (me.getAction() == MotionEvent.ACTION_DOWN) {
			xpos = me.getX();
			ypos = me.getY();
			return true;
		}

		/*
		 * if (me.getAction() == MotionEvent.ACTION_MOVE) { float xd = me.getX()
		 * - xpos; float yd = me.getY() - ypos;
		 * 
		 * xpos = me.getX(); ypos = me.getY();
		 * 
		 * touchTurn = xd / 100f; touchTurnUp = yd / 100f; return true; }
		 */
		return super.onTouchEvent(me);
	}

	public boolean onKeyDown(int keyCode, KeyEvent msg) {

		if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
			zoom = 0.01f;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {
			zoom = -0.01f;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
			turn = 0.03f;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
			turn = -0.03f;
			return true;
		}

		return super.onKeyDown(keyCode, msg);
	}

	public boolean onKeyUp(int keyCode, KeyEvent msg) {
		if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
			zoom = 0;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {
			zoom = 0;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
			turn = 0;
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
			turn = 0;
			return true;
		}

		return super.onKeyUp(keyCode, msg);
	}

	protected boolean isFullscreenOpaque() {
		return true;
	}

	class MyRenderer implements GLSurfaceView.Renderer {

		private FrameBuffer fb = null;
		private Texture font = null;
		private FlagPool flagPool=null;

		private int lfps = 0;
		private int cfps = 0;
		private int fps = 0;

		private Level level = null;
		private AStar aStar = null;
		private Light sun = null;
		private Player player = null;
		private GridPosition[] path = null;
		private int pathPos = 0;
		private SimpleVector target = new SimpleVector();
		private SimpleVector temp = new SimpleVector();
		private SimpleVector temp2 = new SimpleVector();
		private Ticker ticker = new Ticker(25);

		private long time = System.currentTimeMillis();

		private boolean init = false;

		private boolean stop = false;

		private float ind = 0;

		public MyRenderer() {
			Config.maxPolysVisible = 100;
			Config.farPlane = 1500;
			Config.nearPlane = 1;
		}

		public void stop() {
			stop = true;
		}

		public void onSurfaceChanged(GL10 gl, int w, int h) {
			Log.i("onSurfaceChanged", "width: " + w + "/height: " + h);
			if (fb != null) {
				fb.dispose();
				TextureManager.getInstance().flush();
			}
			fb = new FrameBuffer(gl, w, h);
		}

		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			if (!init) {

				TextureManager.getInstance().flush();
				
				init = true;
				world = new World();
				Resources res = getResources();

				Texture.defaultTo4bpp(true);

				TextureManager tm = TextureManager.getInstance();
				tm.addTexture(LevelLoader.WALL_TEXTURE, new Texture(res.openRawResource(R.raw.wall)));
				tm.addTexture(LevelLoader.WALL_TEXTURE_DARK, new Texture(res.openRawResource(R.raw.wall_dark)));
				tm.addTexture(LevelLoader.TOP_TEXTURE, new Texture(res.openRawResource(R.raw.top)));
				tm.addTexture(LevelLoader.FLOOR_TEXTURE, new Texture(res.openRawResource(R.raw.floor)));
				tm.addTexture(LevelLoader.BORDER_TEXTURE, new Texture(res.openRawResource(R.raw.border)));
				tm.addTexture(LevelLoader.FLOOR_TEXTURE + "2", new Texture(res.openRawResource(R.raw.floor2)));
				tm.addTexture(LevelLoader.FLOOR_TEXTURE + "3", new Texture(res.openRawResource(R.raw.floor3)));
				tm.addTexture(LevelLoader.FLOOR_TEXTURE + "4", new Texture(res.openRawResource(R.raw.floor4)));
				
				tm.addTexture("player", new Texture(res.openRawResource(R.raw.player)));
				tm.addTexture("flag", new Texture(res.openRawResource(R.raw.flag)));
				flagPool=new FlagPool(world);
				
				font = new Texture(res.openRawResource(R.raw.numbers));

				try {
					level = new Level(res);
				} catch (Exception e) {
					e.printStackTrace();
				}
				level.addToWorld(world);

				player = new Player(res);
				world.addObject(player);
				player.translate(level.getCenter());

				world.setAmbientLight(80, 80, 80);
				long ms = System.currentTimeMillis();
				world.buildAllObjects();

				Log.i("Building took: ", (System.currentTimeMillis() - ms) + "ms");

				Camera cam = world.getCamera();
				SimpleVector pos = new SimpleVector(level.getCenter());
				pos.y -= 200;
				cam.setPosition(pos);
				cam.lookAt(level.getCenter());

				sun = new Light(world);
				sun.setIntensity(255, 255, 255);
				pos.x += 200;
				sun.setPosition(pos);

				aStar = new AStar(level.getMask());
			}
		}

		public void onDrawFrame(GL10 gl) {

			if (!stop) {

				int ticks = ticker.getTicks();
				Camera cam = world.getCamera();

				if (xpos != -1) {
					SimpleVector pos = Interact2D.reproject2D3D(cam, fb, (int) xpos, (int) ypos);
					if (pos != null) {
						pos.matMul(cam.getBack().invert3x3());
						pos.normalize();

						SimpleVector cPos = cam.getPosition();
						float yd = (level.getCenter().y - cPos.y) / pos.y;

						pos.y = level.getCenter().y;
						pos.x = cPos.x + pos.x * yd;
						pos.z = cPos.z + pos.z * yd;

						GridPosition gp = level.getMask().getGrid(pos.x, pos.z);
						if (level.getMask().isObstacle(gp.getX(), gp.getZ())) {
							// Do nothing
						} else {
							// player.clearTranslation();
							// player.translate(gp.convertTo3D());
							aStar.clear();
							SimpleVector pt = player.getTranslation();
							AIGridPosition ai = aStar.getPathToTarget(level.getMask().getGrid(pt.x, pt.z), gp);
							path = aStar.getPath(ai);
							
							flagPool.clear();
							int end=path.length;
							for (int i=0; i<end; i++) {
								flagPool.add(path[i]);
							}
							
							pathPos = 0;
							if (path.length > 0) {
								path[0].convertTo3D(target);
							}
						}
					}
					xpos = -1;
				}

				if (ticks > 0) {
					// Move player
					if (path != null && pathPos < path.length) {
						temp.set(target);
						temp2 = player.getTranslation(temp2);
						temp.y = temp2.y;
						temp2.scalarMul(-1);

						temp.add(temp2);
						float len = temp.length();
						temp.scalarMul(ticks);
						player.translate(temp.x / len, 0, temp.z / len);

						if (len < 1 * ticks) {
							flagPool.clear(path[pathPos]);
							pathPos++;
							player.clearTranslation();
							player.translate(target.x, -5.5f, target.z);
							if (pathPos < path.length) {
								path[pathPos].convertTo3D(target);
							}
						}
					}

					if (turn != 0) {
						player.rotateDirection(turn * ticks);
					}

					if (zoom != 0) {
						player.zoom(-zoom * ticks);
					}

					player.alignCamera(cam, ticks);
				}

				fb.clear();
				world.renderScene(fb);
				world.draw(fb);
				blitNumber(cfps, 0, 0);

				ind += 0.005f;

				if (ind > 1) {
					ind = 0;
				}

				fb.display();

				if (System.currentTimeMillis() - time >= 1000) {
					Log.i("FPS: ", String.valueOf(fps));
					lfps = (fps + lfps) >> 1;
					cfps = fps;
					fps = 0;
					time = System.currentTimeMillis();
				}
				fps++;
			} else {
				if (fb != null) {
					fb.dispose();
					fb = null;
				}
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		private void blitNumber(int number, int x, int y) {
			if (font != null) {
				String sNum = Integer.toString(number);
				for (int i = 0; i < sNum.length(); i++) {
					char cNum = sNum.charAt(i);
					int iNum = cNum - 48;
					fb.blit(font, iNum * 5, 0, x, y, 5, 9, FrameBuffer.TRANSPARENT_BLITTING);
					x += 5;
				}
			}
		}
	}

	private static class Ticker {

		private int rate;
		private long s2;

		public static long getTime() {
			return System.nanoTime() / 1000000L;
		}

		public Ticker(int tickrateMS) {
			rate = tickrateMS;
			s2 = Ticker.getTime();
		}

		public int getTicks() {
			long i = Ticker.getTime();
			if (i - s2 > rate) {
				int ticks = (int) ((i - s2) / (long) rate);
				s2 += (long) rate * ticks;
				return ticks;
			}
			return 0;
		}
	}

}